from django.db import models
from django.contrib.auth.models import User
from django.db.models.signals import post_save
from django.dispatch import receiver


class Guild(models.Model):
    """
    This model is used to store the guild data.
    """

    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=200)
    tag = models.CharField(max_length=10)
    creation_date = models.DateTimeField("date created", auto_now_add=True)

    def __str__(self):
        return f"{self.name} [{self.tag}]"

    class Meta:
        verbose_name = "Guild"
        verbose_name_plural = "Guilds"


class Player(models.Model):
    """
    This model is used to store the player data.
    Players have a one to one relationship with users.
    """

    id = models.AutoField(primary_key=True)
    user = models.OneToOneField(User, on_delete=models.CASCADE, related_name="player")
    name = models.CharField(max_length=200)
    creation_date = models.DateTimeField("date created", auto_now_add=True)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = "Player"
        verbose_name_plural = "Players"


# Signal to create/update player when user is created/updated
@receiver(post_save, sender=User)
def create_or_update_user_player(sender, instance, created, **kwargs):
    if created:
        Player.objects.create(user=instance, name=instance.username)  # Add default values for other fields if necessary
    else:
        instance.player.save()  # Save the player instance if the user is updated


class PlayerGuild(models.Model):
    """
    This model is used to store the player's guild data.
    """

    id = models.AutoField(primary_key=True)
    player = models.ForeignKey(Player, on_delete=models.CASCADE, related_name="guilds")
    guild = models.ForeignKey(Guild, on_delete=models.CASCADE, related_name="players")
    role = models.CharField(
        max_length=200, default="member"
    )  # admin, officer, member, etc. eventually tie to permissions and account stuff
    creation_date = models.DateTimeField("date created", auto_now_add=True)

    def __str__(self):
        return f"{self.player} - {self.guild}"

    class Meta:
        verbose_name = "Player Guild"
        verbose_name_plural = "Player Guilds"


class Game(models.Model):
    """
    This model is used to store the game data.
    """

    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=200)
    creation_date = models.DateTimeField("date created", auto_now_add=True)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = "Game"
        verbose_name_plural = "Games"


class PlayerGameData(models.Model):
    """
    This model is used to store the game data for a player.
    Anything can be stored in the data field, but it will
    be validated against the schema before being stored.
    """

    id = models.AutoField(primary_key=True)
    player = models.ForeignKey(
        Player, on_delete=models.CASCADE, related_name="game_profiles"
    )
    schema = models.ForeignKey(
        "GuildGameDataSchema",
        on_delete=models.CASCADE,
        related_name="player_game_data",
    )
    creation_date = models.DateTimeField("date created", auto_now_add=True)
    data = models.JSONField()

    def __str__(self):
        return f"{self.player} - {self.game}"

    class Meta:
        verbose_name = "Player Game Data"
        verbose_name_plural = "Player Game Data"


class GuildGameDataSchema(models.Model):
    """
    This model is used to store the schema for the game data.
    Right now it is associated with guild until we determine
    which entity should own the schema.

    Player Game Data will be validated against this schema.
    """

    id = models.AutoField(primary_key=True)
    guild = models.ForeignKey(Guild, on_delete=models.CASCADE, related_name="schemas")
    game = models.ForeignKey(Game, on_delete=models.CASCADE, related_name="schemas")
    schema = models.JSONField()

    def __str__(self):
        return f"{self.guild} - {self.game}"

    class Meta:
        verbose_name = "Guild Game Data Schema"
        verbose_name_plural = "Guild Game Data Schemas"


class WowClasses(models.Model):
    """
    Example of a model that could be used to pre-populate
    dropdowns in the UI.
    """

    id = models.AutoField(primary_key=True)
    name = models.CharField(max_length=200)
    creation_date = models.DateTimeField("date created", auto_now_add=True)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = "WoW Class"
        verbose_name_plural = "WoW Classes"
