from jsonschema import validate
from jsonschema.exceptions import ValidationError
from rest_framework import serializers
from .models import (
    Guild,
    Player,
    Game,
    PlayerGuild,
    PlayerGameData,
    GuildGameDataSchema,
    WowClasses,
)


class GuildSerializer(serializers.ModelSerializer):
    class Meta:
        model = Guild
        fields = ["id", "name", "tag", "creation_date"]
        read_only_fields = ["id", "creation_date"]


class PlayerSerializer(serializers.ModelSerializer):
    class Meta:
        model = Player
        fields = ["id", "name", "creation_date"]
        read_only_fields = ["id", "creation_date"]


class GameSerializer(serializers.ModelSerializer):
    class Meta:
        model = Game
        fields = ["id", "name", "creation_date"]
        read_only_fields = ["id", "creation_date"]


class PlayerGameDataSerializer(serializers.ModelSerializer):
    class Meta:
        model = PlayerGameData
        fields = ["id", "player", "schema", "creation_date", "data"]
        read_only_fields = ["id", "creation_date"]

    def validate_data(self, value):
        try:
            # get the schema for the player's guild and game
            schema = GuildGameDataSchema.objects.get(
                guild=self.context["guild"], game=self.context["game"]
            ).schema

            validate(value, schema)

        except GuildGameDataSchema.DoesNotExist:
            raise serializers.ValidationError(
                "Schema does not exist for this guild and game"
            )
        except ValidationError as e:
            raise serializers.ValidationError(e.message)

        return value


class GuildGameDataSchemaSerializer(serializers.ModelSerializer):
    class Meta:
        model = GuildGameDataSchema
        fields = ["id", "guild", "game", "schema"]
        read_only_fields = ["id"]
    
    def validate_schema(self, value):
        # ensure schema has no more than 15 fields
        # this is just an example it wont actually be this simple
        if len(value["properties"]) > 15:
            raise serializers.ValidationError("Schema cannot have more than 15 fields")


class PlayerGuildSerializer(serializers.ModelSerializer):
    class Meta:
        model = PlayerGuild
        fields = ["id", "player", "guild", "creation_date"]
        read_only_fields = ["id", "creation_date"]


class WowClassesSerializer(serializers.ModelSerializer):
    class Meta:
        model = WowClasses
        fields = ["id", "name", "creation_date"]
        read_only_fields = ["id", "creation_date"]


class LoginSerializer(serializers.Serializer):
    username = serializers.CharField(required=True)
    password = serializers.CharField(write_only=True, required=True)